# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import re
import os
import subprocess

oldArguments = []
newArguments = []
allowedMissingArguments = {}
allowedMissingArguments['devops extension uninstall'] = ['--extension-name', '--publisher-name']
allowedMissingArguments['devops extension install'] = ['--extension-name', '--publisher-name']
allowedMissingArguments['devops extension enable'] = ['--extension-name', '--publisher-name']
allowedMissingArguments['devops extension disable'] = ['--extension-name', '--publisher-name']
allowedMissingArguments['devops extension show'] = ['--extension-name', '--publisher-name']

allowedNewMandatoryArguments = {}
allowedNewMandatoryArguments['devops extension uninstall'] = ['--extension-id', '--publisher-id']
allowedNewMandatoryArguments['devops extension install'] = ['--extension-id', '--publisher-id']
allowedNewMandatoryArguments['devops extension enable'] = ['--extension-id', '--publisher-id']
allowedNewMandatoryArguments['devops extension disable'] = ['--extension-id', '--publisher-id']
allowedNewMandatoryArguments['devops extension show'] = ['--extension-id', '--publisher-id']


# Do not compare these commands
ignoreCommands = []

class Arguments(dict):
    def __init__(self, command, name, isRequired):
        self.command = command
        self.name = name
        self.isRequired = isRequired
        dict.__init__(self,command = command, name = name, isRequired = isRequired)

def extractArgumentsFromCommand(command):
    print('running extractArgumentsFromCommand for ' + command)
    argumentList = []
    commandExtended = 'az ' + command + ' -h'
    help_text = subprocess.run(commandExtended.split(' '), shell=True, stdout=subprocess.PIPE)
    print('help text for ' + command)
    print(help_text)
    if " is in preview" in str(help_text):
        return argumentList
    regexReesult = re.search('Arguments(.*)Global Arguments',str(help_text))
    result = regexReesult.group(1)
    argumentLines = result.split('\\r\\n')

    for argumentLine in argumentLines:
        argumentLineSplits = argumentLine.split(" : ")
        if len(argumentLineSplits) > 1 and ' : ' in argumentLine:
            isRequired = False
            if '[Required]' in argumentLineSplits[0]:
                isRequired = True

            names = argumentLineSplits[0].replace('[Required]','').strip().split(' ')
            for name in names:
                argument = Arguments(command, name, isRequired)
                argumentList.append(argument)

    return argumentList

# remove azure-devops extension from index (if installed)
subprocess.run(['az', 'extension', 'remove', '-n', 'azure-devops'], shell=True)

# install extension from index
subprocess.run(['az', 'extension', 'add', '-n', 'azure-devops'], shell=True)

# Check the installed extensions
subprocess.run(['az', 'extension', 'list'], shell=True)

# add extension path to sys.path so that we can get all the commands
import sys
from azure.cli.core.extension import get_extension_path
# Make sure that the extension install directory is on sys.path so that dependencies can be found.
extensionPath = get_extension_path('azure-devops')
sys.path.append(extensionPath)

# loading commands from code
from azure.cli.core.mock import DummyCli
from azext_devops import DevCommandsLoader
cli_ctx = DummyCli()
loader = DevCommandsLoader(cli_ctx)
loader.load_command_table(None)

for command in loader.command_table:
    oldArguments.extend(extractArgumentsFromCommand(command))

print('Unload extension (loaded from index).')

# uninstall extension loaded from index
subprocess.run(['az', 'extension', 'remove', '-n', 'azure-devops'], shell=True, stdout=subprocess.PIPE)

# search and install extension from given path
def findExtension():
    for p, d, f in os.walk('.'):
        for file in f:
            if file.endswith('.whl'):
                return os.path.join(p, file)


newExtensionLocation = findExtension()
print('Install extension (loaded from current code). Wheel path - {}'.format(newExtensionLocation))
subprocess.run(['az', 'extension', 'add', '--source', newExtensionLocation, '-y'], shell=True, stdout=subprocess.PIPE)

# Check the installed extensions
subprocess.run(['az', 'extension', 'list'], shell=True)

# get a set of old commands, we are not reusing the set from ext because we want to keep this clean
oldCommands = []
for oldArgument in oldArguments:
    if oldArgument.command not in ignoreCommands:
        if not (oldArgument.command in oldCommands):
            oldCommands.append(oldArgument.command)
    else:
        print('Ignoring command.. ' + oldArgument.command)


# prepare argument set from new extension
for oldCommand in oldCommands:
    print("Running extract for command: {}".format(oldCommand))
    newArguments.extend(extractArgumentsFromCommand(oldCommand))

errorList = []

# make sure no new argument is mandatory
for newArgument in newArguments:
    if newArgument.isRequired is True:
        isNewMandatory = True
        for oldArgument in oldArguments:
            if oldArgument.command == newArgument.command and oldArgument.name == newArgument.name and oldArgument.isRequired is True:
                isNewMandatory = False
                break

        if isNewMandatory is True:
            allowedNewMandatoryArgumentsForCommand = allowedNewMandatoryArguments.get(newArgument.command, [])
            if not newArgument.name in allowedNewMandatoryArgumentsForCommand:
                errorList.append('\n' + 'New Mandatory argument found for command ' + newArgument.command + ' argument ' +  newArgument.name)

# make sure no argument is removed
for oldArgument in oldArguments:
    if oldArgument.command not in ignoreCommands:
        isArgumentMissing = True
        for newArgument in newArguments:
            if oldArgument.name == newArgument.name and oldArgument.command == newArgument.command:
                isArgumentMissing = False
                break

        if isArgumentMissing is True:
            allowedMissingArgumetsForCommand = allowedMissingArguments.get(oldArgument.command, [])
            if not oldArgument.name in allowedMissingArgumetsForCommand:
                errorList.append('\n' + 'Argument missing for command ' + oldArgument.command + ' argument ' +  oldArgument.name)

if len(errorList) > 0:
    import sys
    sys.stderr.write(' '.join(errorList))
    raise Exception('Something is not correct')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class FeatureAvailabilityClient(Client):
    """FeatureAvailability
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(FeatureAvailabilityClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = None

    def get_all_feature_flags(self, user_email=None):
        """GetAllFeatureFlags.
        [Preview API] Retrieve a listing of all feature flags and their current states for a user
        :param str user_email: The email of the user to check
        :rtype: [FeatureFlag]
        """
        query_parameters = {}
        if user_email is not None:
            query_parameters['userEmail'] = self._serialize.query('user_email', user_email, 'str')
        response = self._send(http_method='GET',
                              location_id='3e2b80f8-9e6f-441e-8393-005610692d9c',
                              version='6.0-preview.1',
                              query_parameters=query_parameters)
        return self._deserialize('[FeatureFlag]', self._unwrap_collection(response))

    def get_feature_flag_by_name(self, name, check_feature_exists=None):
        """GetFeatureFlagByName.
        [Preview API] Retrieve information on a single feature flag and its current states
        :param str name: The name of the feature to retrieve
        :param bool check_feature_exists: Check if feature exists
        :rtype: :class:`<FeatureFlag> <azure.devops.v6_0.feature_availability.models.FeatureFlag>`
        """
        route_values = {}
        if name is not None:
            route_values['name'] = self._serialize.url('name', name, 'str')
        query_parameters = {}
        if check_feature_exists is not None:
            query_parameters['checkFeatureExists'] = self._serialize.query('check_feature_exists', check_feature_exists, 'bool')
        response = self._send(http_method='GET',
                              location_id='3e2b80f8-9e6f-441e-8393-005610692d9c',
                              version='6.0-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('FeatureFlag', response)

    def get_feature_flag_by_name_and_user_email(self, name, user_email, check_feature_exists=None):
        """GetFeatureFlagByNameAndUserEmail.
        [Preview API] Retrieve information on a single feature flag and its current states for a user
        :param str name: The name of the feature to retrieve
        :param str user_email: The email of the user to check
        :param bool check_feature_exists: Check if feature exists
        :rtype: :class:`<FeatureFlag> <azure.devops.v6_0.feature_availability.models.FeatureFlag>`
        """
        route_values = {}
        if name is not None:
            route_values['name'] = self._serialize.url('name', name, 'str')
        query_parameters = {}
        if user_email is not None:
            query_parameters['userEmail'] = self._serialize.query('user_email', user_email, 'str')
        if check_feature_exists is not None:
            query_parameters['checkFeatureExists'] = self._serialize.query('check_feature_exists', check_feature_exists, 'bool')
        response = self._send(http_method='GET',
                              location_id='3e2b80f8-9e6f-441e-8393-005610692d9c',
                              version='6.0-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('FeatureFlag', response)

    def get_feature_flag_by_name_and_user_id(self, name, user_id, check_feature_exists=None):
        """GetFeatureFlagByNameAndUserId.
        [Preview API] Retrieve information on a single feature flag and its current states for a user
        :param str name: The name of the feature to retrieve
        :param str user_id: The id of the user to check
        :param bool check_feature_exists: Check if feature exists
        :rtype: :class:`<FeatureFlag> <azure.devops.v6_0.feature_availability.models.FeatureFlag>`
        """
        route_values = {}
        if name is not None:
            route_values['name'] = self._serialize.url('name', name, 'str')
        query_parameters = {}
        if user_id is not None:
            query_parameters['userId'] = self._serialize.query('user_id', user_id, 'str')
        if check_feature_exists is not None:
            query_parameters['checkFeatureExists'] = self._serialize.query('check_feature_exists', check_feature_exists, 'bool')
        response = self._send(http_method='GET',
                              location_id='3e2b80f8-9e6f-441e-8393-005610692d9c',
                              version='6.0-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('FeatureFlag', response)

    def update_feature_flag(self, state, name, user_email=None, check_feature_exists=None, set_at_application_level_also=None):
        """UpdateFeatureFlag.
        [Preview API] Change the state of an individual feature flag for a name
        :param :class:`<FeatureFlagPatch> <azure.devops.v6_0.feature_availability.models.FeatureFlagPatch>` state: State that should be set
        :param str name: The name of the feature to change
        :param str user_email:
        :param bool check_feature_exists: Checks if the feature exists before setting the state
        :param bool set_at_application_level_also:
        :rtype: :class:`<FeatureFlag> <azure.devops.v6_0.feature_availability.models.FeatureFlag>`
        """
        route_values = {}
        if name is not None:
            route_values['name'] = self._serialize.url('name', name, 'str')
        query_parameters = {}
        if user_email is not None:
            query_parameters['userEmail'] = self._serialize.query('user_email', user_email, 'str')
        if check_feature_exists is not None:
            query_parameters['checkFeatureExists'] = self._serialize.query('check_feature_exists', check_feature_exists, 'bool')
        if set_at_application_level_also is not None:
            query_parameters['setAtApplicationLevelAlso'] = self._serialize.query('set_at_application_level_also', set_at_application_level_also, 'bool')
        content = self._serialize.body(state, 'FeatureFlagPatch')
        response = self._send(http_method='PATCH',
                              location_id='3e2b80f8-9e6f-441e-8393-005610692d9c',
                              version='6.0-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters,
                              content=content)
        return self._deserialize('FeatureFlag', response)




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .symbol_client import SymbolClient

__all__ = [
    'DebugEntry',
    'DebugEntryCreateBatch',
    'JsonBlobBlockHash',
    'JsonBlobIdentifier',
    'JsonBlobIdentifierWithBlocks',
    'Request',
    'ResourceBase',
    'SymbolClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from .models import *
from .task_agent_client import TaskAgentClient

__all__ = [
    'AadOauthTokenRequest',
    'AadOauthTokenResult',
    'AuthenticationSchemeReference',
    'AuthorizationHeader',
    'AzureManagementGroup',
    'AzureManagementGroupQueryResult',
    'AzureSubscription',
    'AzureSubscriptionQueryResult',
    'ClientCertificate',
    'DataSource',
    'DataSourceBinding',
    'DataSourceBindingBase',
    'DataSourceDetails',
    'DependencyBinding',
    'DependencyData',
    'DependsOn',
    'DeploymentGroup',
    'DeploymentGroupCreateParameter',
    'DeploymentGroupCreateParameterPoolProperty',
    'DeploymentGroupMetrics',
    'DeploymentGroupReference',
    'DeploymentGroupUpdateParameter',
    'DeploymentMachine',
    'DeploymentMachineGroup',
    'DeploymentMachineGroupReference',
    'DeploymentPoolSummary',
    'DeploymentTargetUpdateParameter',
    'EndpointAuthorization',
    'EndpointUrl',
    'EnvironmentCreateParameter',
    'EnvironmentDeploymentExecutionRecord',
    'EnvironmentInstance',
    'EnvironmentReference',
    'EnvironmentResource',
    'EnvironmentResourceReference',
    'EnvironmentUpdateParameter',
    'GraphSubjectBase',
    'HelpLink',
    'IdentityRef',
    'InputDescriptor',
    'InputValidation',
    'InputValidationRequest',
    'InputValue',
    'InputValues',
    'InputValuesError',
    'KubernetesResource',
    'KubernetesResourceCreateParameters',
    'MetricsColumnMetaData',
    'MetricsColumnsHeader',
    'MetricsRow',
    'PackageMetadata',
    'PackageVersion',
    'ProjectReference',
    'PublishTaskGroupMetadata',
    'ReferenceLinks',
    'ResourceLimit',
    'ResourceUsage',
    'ResultTransformationDetails',
    'SecureFile',
    'ServiceEndpoint',
    'ServiceEndpointAuthenticationScheme',
    'ServiceEndpointDetails',
    'ServiceEndpointExecutionData',
    'ServiceEndpointExecutionRecord',
    'ServiceEndpointExecutionRecordsInput',
    'ServiceEndpointRequest',
    'ServiceEndpointRequestResult',
    'ServiceEndpointType',
    'TaskAgent',
    'TaskAgentAuthorization',
    'TaskAgentCloud',
    'TaskAgentCloudRequest',
    'TaskAgentCloudType',
    'TaskAgentDelaySource',
    'TaskAgentJobRequest',
    'TaskAgentMessage',
    'TaskAgentPool',
    'TaskAgentPoolMaintenanceDefinition',
    'TaskAgentPoolMaintenanceJob',
    'TaskAgentPoolMaintenanceJobTargetAgent',
    'TaskAgentPoolMaintenanceOptions',
    'TaskAgentPoolMaintenanceRetentionPolicy',
    'TaskAgentPoolMaintenanceSchedule',
    'TaskAgentPoolReference',
    'TaskAgentPublicKey',
    'TaskAgentQueue',
    'TaskAgentReference',
    'TaskAgentSession',
    'TaskAgentSessionKey',
    'TaskAgentUpdate',
    'TaskAgentUpdateReason',
    'TaskDefinition',
    'TaskDefinitionEndpoint',
    'TaskDefinitionReference',
    'TaskExecution',
    'TaskGroup',
    'TaskGroupCreateParameter',
    'TaskGroupDefinition',
    'TaskGroupRevision',
    'TaskGroupStep',
    'TaskGroupUpdateParameter',
    'TaskGroupUpdatePropertiesBase',
    'TaskHubLicenseDetails',
    'TaskInputDefinition',
    'TaskInputDefinitionBase',
    'TaskInputValidation',
    'TaskOrchestrationOwner',
    'TaskOutputVariable',
    'TaskPackageMetadata',
    'TaskReference',
    'TaskSourceDefinition',
    'TaskSourceDefinitionBase',
    'TaskVersion',
    'ValidationItem',
    'VariableGroup',
    'VariableGroupParameters',
    'VariableGroupProjectReference',
    'VariableGroupProviderData',
    'VariableValue',
    'VirtualMachine',
    'VirtualMachineGroup',
    'VirtualMachineGroupCreateParameters',
    'TaskAgentClient'
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help import GroupHelpFile
from azure.cli.core._help import  CliCommandHelpFile
from azure.cli.core.mock import DummyCli
from azext_devops import DevCommandsLoader


def get_extension_help_files(cli_ctx):
    loader = DevCommandsLoader(cli_ctx)
    cmd_table = loader.load_command_table(None)
    invoker = cli_ctx.invocation_cls(cli_ctx=cli_ctx, commands_loader_cls=cli_ctx.commands_loader_cls,
                                     parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
    cli_ctx.invocation = invoker
    invoker.commands_loader.command_table = cmd_table

    for cmd_name in cmd_table:
        invoker.commands_loader.load_arguments(cmd_name)
    invoker.parser.load_command_table(invoker.commands_loader)

    parser_keys = []
    parser_values = []
    sub_parser_keys = []
    sub_parser_values = []
    _store_parsers(invoker.parser, parser_keys, parser_values, sub_parser_keys, sub_parser_values)
    for cmd, parser in zip(parser_keys, parser_values):
        if cmd not in sub_parser_keys:
            sub_parser_keys.append(cmd)
            sub_parser_values.append(parser)
    help_ctx = cli_ctx.help_cls(cli_ctx=cli_ctx)
    help_files = []
    for cmd, parser in zip(sub_parser_keys, sub_parser_values):
        try:
            help_file = GroupHelpFile(help_ctx, cmd, parser) if _is_group(parser) \
                else CliCommandHelpFile(help_ctx, cmd, parser)
            help_file.load(parser)
            help_files.append(help_file)
        except Exception as ex:
            print("Skipped '{}' due to '{}'".format(cmd, ex))
    help_files = sorted(help_files, key=lambda x: x.command)
    return help_files

def _store_parsers(parser, parser_keys, parser_values, sub_parser_keys, sub_parser_values):
    for s in parser.subparsers.values():
        parser_keys.append(_get_parser_name(s))
        parser_values.append(s)
        if _is_group(s):
            for c in s.choices.values():
                sub_parser_keys.append(_get_parser_name(c))
                sub_parser_values.append(c)
                _store_parsers(c, parser_keys, parser_values, sub_parser_keys, sub_parser_values)

def _is_group(parser):
    return getattr(parser, '_subparsers', None) is not None \
        or getattr(parser, 'choices', None) is not None

def _get_parser_name(s):
    return (s._prog_prefix if hasattr(s, '_prog_prefix') else s.prog)[3:]

def print_missing_help_files(help_files):
    missing_help_text = False
    missing_help_list = []
    for help_file in help_files:
        if help_file.short_summary == '' and help_file.command != '':
            is_command = isinstance(help_file, CliCommandHelpFile)
            command_type = 'command' if is_command else 'group'
            missing_help_list.append((command_type, help_file.command))
            missing_help_text = True
    if not missing_help_text:
        print('No missing help texts found.')
    else:
        print('\n\nNo help texts were found for below command(s):')
        for text in missing_help_list :
            print('{} : {}'.format(text[0], text[1]))
        raise Exception('Please update the help text(s).')


azure_devops_cli_ctx = DummyCli()
helpfiles = get_extension_help_files(azure_devops_cli_ctx)
print_missing_help_files(helpfiles)



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function

import sys

from knack.log import get_logger
from knack.prompting import NoTTYException, prompt_pass
from knack.util import CLIError
from msrest.authentication import BasicAuthentication
from azext_devops.dev.common._credentials import clear_credential, set_credential
from azext_devops.dev.common.services import _get_connection, get_base_url

logger = get_logger(__name__)


def credential_set(organization=None):
    """Set the credential (PAT) to use for a particular organization.
    Refer https://aka.ms/azure-devops-cli-auth for more information on providing PAT as input.
    """
    token = _get_pat_token()
    if organization is not None:
        organization = get_base_url(organization)
        logger.info("Creating connection with personal access token.")
        _verify_token(organization=organization, token=token)
    try:
        set_credential(organization=organization, token=token)
    except Exception as ex:  # pylint: disable=bare-except
        logger.warning("Unable to use secure credential store in this environment.")
        logger.warning("Please refer to alternate methods at https://aka.ms/azure-devops-cli-auth")
        logger.warning("using Environment variable")
        logger.warning("or use 'az login'")
        raise CLIError(ex)
    _check_and_set_default_organization(organization)


def credential_clear(organization=None):
    """Clear the credential for all or a particular organization
    :param organization: Azure Devops organization URL. Example: https://dev.azure.com/MyOrganizationName/.
    If no organization is specified, all organizations will be logged out.
    :type organization: str
    """
    if organization is not None:
        organization = get_base_url(organization)
    clear_credential(organization)
    if organization is not None:
        print('The credential was successfully cleared.')
    else:
        print('Logged out of all Azure DevOps organizations.')
    _check_and_clear_default_organization(organization)


def _verify_token(organization, token):
    credentials = BasicAuthentication('', token)
    connection = _get_connection(organization, credentials)
    vstsDir = 'azext_devops.devops_sdk.'
    location_client = connection.get_client(vstsDir + 'v5_0.location.location_client.LocationClient')
    try:
        connection_data = location_client.get_connection_data()
    except Exception as ex2:
        logger.debug(ex2, exc_info=True)
        raise CLIError("Failed to authenticate using the supplied token.")
    # An organization with public project enabled will not throw any exception for invalid token.
    # Hence, handle anonymous user case here.
    if connection_data.authenticated_user.id == _ANONYMOUS_USER_ID:
        raise CLIError("Failed to authenticate using the supplied token.")


def _get_pat_token():
    try:
        token = prompt_pass('Token: ', confirm=False, help_string="The token (PAT) to authenticate with.")
        while len(token) <= 1:
            logger.warning('Please provide a PAT token.')
            logger.warning('If you are using CTRL + V to paste the token, it won\'t work '
                           'on windows command prompt or powershell - '
                           'https://github.com/microsoft/knack/issues/160.')
            logger.warning('Use right click or console menu to paste the token.')
            token = prompt_pass('Token: ', confirm=False, help_string="The token (PAT) to authenticate with.")
    except NoTTYException:
        logger.info("Getting PAT token in non-interactive mode.")
        token = sys.stdin.readline().rstrip()
    return token


# Sets organization if the default is not set
def _check_and_set_default_organization(organization):
    if organization is not None:
        from azext_devops.dev.common.config import azdevops_config
        from azext_devops.dev.common.const import DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT
        from .configure import configure
        current_org_default = None
        if azdevops_config.has_option(DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT):
            current_org_default = azdevops_config.get(DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT)
        if current_org_default is None or current_org_default == '':
            configure(defaults=['organization={}'.format(organization)])
            logger.debug("Setting this organization as default. No default was set earlier.")
        else:
            logger.debug("Another organization is already set as default.")


# Clears organization if the default is set to same
def _check_and_clear_default_organization(organization):
    if organization is not None:
        from azext_devops.dev.common.config import azdevops_config
        from azext_devops.dev.common.const import DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT
        from .configure import configure
        current_org_default = None
        if azdevops_config.has_option(DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT):
            current_org_default = azdevops_config.get(DEFAULTS_SECTION, DEVOPS_ORGANIZATION_DEFAULT)
        if current_org_default == organization:
            configure(defaults=["organization="])
            logger.debug("Resetting default organization.")
        else:
            logger.debug("Default org not reset. Different organization is set as default.")


_ANONYMOUS_USER_ID = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure_devtools.scenario_tests import AllowLargeResponse
from .utilities.helper import DevopsScenarioTest, disable_telemetry, set_authentication, get_test_org_from_env_variable

DEVOPS_CLI_TEST_ORGANIZATION = get_test_org_from_env_variable() or 'Https://dev.azure.com/azuredevopsclitest'

class PipelinesFolderTests(DevopsScenarioTest):
    @AllowLargeResponse(size_kb=3072)
    @disable_telemetry
    @set_authentication
    def test_pipeline_folders(self):
        random_project_name = self.create_random_name(prefix='pipelinesFolderTest', length=25)
        self.cmd('az devops configure --defaults organization=' +  DEVOPS_CLI_TEST_ORGANIZATION + ' project=' + random_project_name)
        created_project_id = None

        try:
            create_project_command = 'az devops project create --name ' + random_project_name + ' --output json --detect false'
            project_create_output = self.cmd(create_project_command).get_output_in_json()
            created_project_id = project_create_output["id"]

            # create a pipeline
            FOLDER_NAME = '\\TestFolder'
            FOLDER_DESCRIPTION = 'Test folder'
            folder_create_command = 'az pipelines folder create --path {} --description "{}" \
                --detect false --output json'.format(FOLDER_NAME, FOLDER_DESCRIPTION)
            folder_create_output = self.cmd(folder_create_command).get_output_in_json()
            assert folder_create_output['path'] == FOLDER_NAME
            assert folder_create_output['description'] == FOLDER_DESCRIPTION

            UPDATED_FOLDER_DESCRIPTION = 'New test folder'
            UPDATED_FOLDER_PATH = '\\Testing' + FOLDER_NAME
            folder_update_command = 'az pipelines folder update --path {} --new-path "{}" --new-description "{}" \
                --detect false --output json'.format(FOLDER_NAME, UPDATED_FOLDER_PATH, UPDATED_FOLDER_DESCRIPTION)
            folder_update_output = self.cmd(folder_update_command).get_output_in_json()
            assert folder_update_output['path'] == UPDATED_FOLDER_PATH
            assert folder_update_output['description'] == UPDATED_FOLDER_DESCRIPTION

            folder_list_command = 'az pipelines folder list --detect false --output json --query-order asc'
            folder_list_output = self.cmd(folder_list_command).get_output_in_json()
            assert len(folder_list_output) == 3  # root folder + testing/test123 nested folders
            assert folder_list_output[0]['path'] == '\\'  # root folder
            assert folder_list_output[0]['description'] == None
            assert folder_list_output[1]['path'] == '\\Testing'
            assert folder_list_output[1]['description'] == None
            assert folder_list_output[2]['path'] == UPDATED_FOLDER_PATH
            assert folder_list_output[2]['description'] == UPDATED_FOLDER_DESCRIPTION

            folder_delete_command = 'az pipelines folder delete --path "{}" --detect false --output json -y'.format(UPDATED_FOLDER_PATH)
            folder_delete_output = self.cmd(folder_delete_command)
            # verify deletion
            folder_list_output = self.cmd(folder_list_command).get_output_in_json()
            assert len(folder_list_output) == 2
            assert folder_list_output[0]['path'] == '\\'  # root folder
            assert folder_list_output[0]['description'] == None
            assert folder_list_output[1]['path'] == '\\Testing'
            assert folder_list_output[1]['description'] == None

        finally:
            if created_project_id is not None:
                delete_project_command = 'az devops project delete --id ' + created_project_id + ' --output json --detect false -y'
                self.cmd(delete_project_command)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import sys
from azure.cli.core.extension import get_extension_path
# Make sure that the extension install directory is on sys.path so that dependencies can be found.

extensionPath = get_extension_path('azure-devops')
sys.path.append(extensionPath)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.arguments import enum_choice_list
from azure.cli.core.commands.parameters import get_enum_type, get_three_state_flag

# CUSTOM CHOICE LISTS
_BRANCH_MATCH_KIND_VALUES = ['prefix', 'exact']
_VOTE_VALUES = ['approve', 'approve-with-suggestions', 'reset', 'wait-for-author', 'reject']
_PR_STATUS_VALUES = ['all', 'active', 'completed', 'abandoned']
_PR_TARGET_STATUS_VALUES = ['active', 'completed', 'abandoned']


# pylint: disable=too-many-statements
def load_code_arguments(self, _):
    with self.argument_context('repos') as context:
        context.argument('repository', options_list=('--repository', '-r'))
        context.argument('reviewers', nargs='*')

    with self.argument_context('repos policy') as context:
        context.argument('policy_configuration', options_list=('--policy-configuration', '--config'),
                         help='Local file path for configuration file. ' +
                         'Please use \\backslash when typing in directory path.')
        context.argument('policy_id', options_list=('--policy-id', '--id'),
                         help='ID of the policy.')
        context.argument('repository_id', help='Id of the repository on which to apply the policy')
        context.argument('branch', help='Branch on which this policy should be applied. For example: master')
        context.argument('branch_match_type', arg_type=get_enum_type(_BRANCH_MATCH_KIND_VALUES),
                         help='Determines how the branch argument is used to apply a policy. ' +
                         'If value is \'exact\', the policy will be applied on a branch which has an ' +
                         'exact match on the --branch argument. ' +
                         'If value is \'prefix\' the policy is applied across all branch folders that ' +
                         'match the prefix provided by the --branch argument.')
        context.argument('blocking', arg_type=get_three_state_flag(),
                         help='Whether the policy should be blocking or not')
        context.argument('enabled', arg_type=get_three_state_flag(),
                         help='Whether the policy is enabled or not')
        context.argument('path_filter',
                         help='Filter path(s) on which the policy is applied. ' +
                         'Supports absolute paths, wildcards and multiple paths separated by \';\'. ' +
                         'Example: /WebApp/Models/Data.cs, /WebApp/* or *.cs,' +
                         '/WebApp/Models/Data.cs;ClientApp/Models/Data.cs')

    with self.argument_context('repos policy list') as context:
        context.argument('branch',
                         help='Branch name to filter results by exact match of branch name. ' +
                         'The --repository-id parameter is required to use the branch filter. ' +
                         'For example: --branch master')
        context.argument('repository_id',
                         help='ID of the repository to filter results by exact match of the repository ID. ' +
                         'For example --repository-ID e556f204-53c9-4153-9cd9-ef41a11e3345')

    with self.argument_context('repos policy approver-count') as context:
        context.argument('minimum_approver_count',
                         help='Minimum number of approvers required. For example: 2')
        context.argument('creator_vote_counts', arg_type=get_three_state_flag(),
                         help='Whether the creator\'s vote counts or not.')
        context.argument('allow_downvotes', arg_type=get_three_state_flag(),
                         help='Whether to allow downvotes or not.')
        context.argument('reset_on_source_push', arg_type=get_three_state_flag(),
                         help='Whether to reset source on push.')

    with self.argument_context('repos policy merge-strategy') as context:
        context.argument('use_squash_merge', arg_type=get_three_state_flag(),
                         help='Whether to squash merge always. '
                         'This option does not work for allowing other merge types.',
                         deprecate_info=context.deprecate(redirect='--allow-squash', target='--use-squash-merge',
                                                          hide=True))
        context.argument('allow_squash', arg_type=get_three_state_flag(),
                         help='Squash merge - Creates a linear history by condensing the source branch commits '
                         'into a single new commit on the target branch.')
        context.argument('allow_rebase', arg_type=get_three_state_flag(),
                         help='Rebase and fast-forward - Creates a linear history by replaying the source branch '
                         'commits onto the target without a merge commit.')
        context.argument('allow_no_fast_forward', arg_type=get_three_state_flag(),
                         help='Basic merge (no fast-forward) - Preserves nonlinear history exactly as it happened '
                         'during development.')
        context.argument('allow_rebase_merge', arg_type=get_three_state_flag(),
                         help='Rebase with merge commit - Creates a semi-linear history by replaying the source '
                         'branch commits onto the target and then creating a merge commit.')

    with self.argument_context('repos policy build') as context:
        context.argument('build_definition_id', help='Build Definition Id.')
        context.argument('queue_on_source_update_only', arg_type=get_three_state_flag(),
                         help='Queue Only on source update.')
        context.argument('manual_queue_only', arg_type=get_three_state_flag(),
                         help='Whether to allow only manual queue of builds.')
        context.argument('display_name',
                         help='Display name for this build policy to identify the policy. ' +
                         'For example: \'Manual queue policy\'')
        context.argument('valid_duration', help='Policy validity duration (in minutes).')

    with self.argument_context('repos policy file-size') as context:
        context.argument('maximum_git_blob_size',
                         help='Maximum git blob size in bytes. ' +
                         'For example, to specify a 10byte limit, --maximum-git-blob-size 10.')
        context.argument('use_uncompressed_size', arg_type=get_three_state_flag(),
                         help='Whether to use uncompressed size.')

    with self.argument_context('repos policy required-reviewer') as context:
        context.argument('required_reviewer_ids',
                         help='Required reviewers email addresses separated by \';\'. ' +
                         'For example: john@contoso.com;alice@contoso.com')
        context.argument('message', help='Message.')

    with self.argument_context('repos pr') as context:
        context.argument('description', type=str, options_list=('--description', '-d'), nargs='*')
        context.argument('source_branch', options_list=('--source-branch', '-s'))
        context.argument('target_branch', options_list=('--target-branch', '-t'))
        context.argument('title', type=str)

    with self.argument_context('repos pr create') as context:
        context.argument('work_items', nargs='*')
        context.argument('draft', arg_type=get_three_state_flag())
        context.argument('auto_complete', arg_type=get_three_state_flag())
        context.argument('squash', arg_type=get_three_state_flag())
        context.argument('delete_source_branch', arg_type=get_three_state_flag())
        context.argument('bypass_policy', arg_type=get_three_state_flag())
        context.argument('transition_work_items', arg_type=get_three_state_flag())

    with self.argument_context('repos pr list') as context:
        context.argument('status', **enum_choice_list(_PR_STATUS_VALUES))

    with self.argument_context('repos pr reviewer') as context:
        context.argument('reviewers', nargs='+')

    with self.argument_context('repos pr work-item') as context:
        context.argument('work_items', nargs='+')

    with self.argument_context('repos pr update') as context:
        context.argument('auto_complete', arg_type=get_three_state_flag())
        context.argument('squash', arg_type=get_three_state_flag())
        context.argument('delete_source_branch', arg_type=get_three_state_flag())
        context.argument('bypass_policy', arg_type=get_three_state_flag())
        context.argument('transition_work_items', arg_type=get_three_state_flag())
        context.argument('draft', arg_type=get_three_state_flag())
        context.argument('status', **enum_choice_list(_PR_TARGET_STATUS_VALUES))

    with self.argument_context('repos pr policy') as context:
        context.argument('evaluation_id', options_list=('--evaluation-id', '-e'))

    with self.argument_context('repos pr set-vote') as context:
        context.argument('vote', **enum_choice_list(_VOTE_VALUES))

    with self.argument_context('repos delete') as context:
        context.argument('yes', options_list=['--yes', '-y'], action='store_true',
                         help='Do not prompt for confirmation.')

    with self.argument_context('repos import create') as context:
        context.argument('git_source_url', options_list=('--git-source-url', '--git-url'))

    with self.argument_context('repos ref') as context:
        context.argument('repository', options_list=('--repository', '-r'))
        context.argument('object_id', options_list=('--object-id'))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch
from azext_devops.dev.common.vsts_git_url_info import VstsGitUrlInfo


class Test_VstsGitUrlInfo_Methods(unittest.TestCase):

    def test_get_vsts_info_old_url_format_with_new_ssh_format(self):
        with patch('azext_devops.devops_sdk.v5_0.git.git_client.GitClient.get_vsts_info_by_remote_url') as mock_get_vsts_info:
            with patch('azext_devops.dev.common.services._get_credentials') as mock_get_creds:
                VstsGitUrlInfo.get_vsts_info('organization@vs-ssh.visualstudio.com:v3/organization/project/repository')
                # Asserts            
                mock_get_vsts_info.assert_called_once()
                mock_get_creds.assert_called_once()
                get_vsts_info_url_param = mock_get_vsts_info.call_args_list[0][0]
                self.assertEqual(
                    'https://organization.visualstudio.com/project/_git/repository'.lower(), get_vsts_info_url_param[0])
                    

    def test_get_vsts_info_old_url_format_with_https(self):
        with patch('azext_devops.devops_sdk.v5_0.git.git_client.GitClient.get_vsts_info_by_remote_url') as mock_get_vsts_info:
            with patch('azext_devops.dev.common.services._get_credentials') as mock_get_creds:
                VstsGitUrlInfo.get_vsts_info('https://organization.visualstudio.com/project/_git/repository')
                # Asserts            
                mock_get_vsts_info.assert_called_once()
                mock_get_creds.assert_called_once()
                get_vsts_info_url_param = mock_get_vsts_info.call_args_list[0][0]
                self.assertEqual(
                    'https://organization.visualstudio.com/project/_git/repository'.lower(), get_vsts_info_url_param[0])
                    

    def test_get_vsts_info_old_url_format_with_old_ssh_format(self):
        with patch('azext_devops.devops_sdk.v5_0.git.git_client.GitClient.get_vsts_info_by_remote_url') as mock_get_vsts_info:
            with patch('azext_devops.dev.common.services._get_credentials') as mock_get_creds:
                VstsGitUrlInfo.get_vsts_info('ssh://organization@vs-ssh.visualstudio.com:22/project/_ssh/repository')
                # Asserts            
                mock_get_vsts_info.assert_called_once()
                mock_get_creds.assert_called_once()
                get_vsts_info_url_param = mock_get_vsts_info.call_args_list[0][0]
                self.assertEqual(
                    'https://organization.visualstudio.com/project/_git/repository'.lower(), get_vsts_info_url_param[0])


    def test_get_vsts_inf_new_url_format_with_ssh(self):
        with patch('azext_devops.devops_sdk.v5_0.git.git_client.GitClient.get_vsts_info_by_remote_url') as mock_get_vsts_info:
            with patch('azext_devops.dev.common.services._get_credentials') as mock_get_creds:
                VstsGitUrlInfo.get_vsts_info('git@ssh.dev.azure.com:v3/organization/project/repository')
                # Asserts            
                mock_get_vsts_info.assert_called_once()
                mock_get_creds.assert_called_once()
                get_vsts_info_url_param = mock_get_vsts_info.call_args_list[0][0]
                self.assertEqual(
                    'https://dev.azure.com/organization/project/_git/repository'.lower(), get_vsts_info_url_param[0])

    def test_get_vsts_inf_new_url_format_with_https(self):
        with patch('azext_devops.devops_sdk.v5_0.git.git_client.GitClient.get_vsts_info_by_remote_url') as mock_get_vsts_info:
            with patch('azext_devops.dev.common.services._get_credentials') as mock_get_creds:
                VstsGitUrlInfo.get_vsts_info('https://organization@dev.azure.com/organization/project/_git/repository')
                # Asserts            
                mock_get_vsts_info.assert_called_once()
                mock_get_creds.assert_called_once()
                get_vsts_info_url_param = mock_get_vsts_info.call_args_list[0][0]
                self.assertEqual(
                    'https://organization@dev.azure.com/organization/project/_git/repository'.lower(), get_vsts_info_url_param[0])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest

from azure_devtools.scenario_tests import AllowLargeResponse
from .utilities.helper import DevopsScenarioTest, disable_telemetry, set_authentication, get_test_org_from_env_variable

DEVOPS_CLI_TEST_ORGANIZATION = get_test_org_from_env_variable() or 'Https://dev.azure.com/azuredevopsclitest'

class PipelinesTests(DevopsScenarioTest):
    @AllowLargeResponse(size_kb=3072)
    @disable_telemetry
    @set_authentication
    def test_variable_group(self):
        random_project_name = self.create_random_name(prefix='varGroupTests', length=20)
        self.cmd('az devops configure --defaults organization=' +  DEVOPS_CLI_TEST_ORGANIZATION + ' project=' + random_project_name)
        created_project_id = None

        try:
            create_project_command = 'az devops project create --name ' + random_project_name + ' --output json --detect false'
            project_create_output = self.cmd(create_project_command).get_output_in_json()
            created_project_id = project_create_output["id"]

            # Create variable group
            create_variable_group_command = 'az pipelines variable-group create --name group1 \
                --variables var1=value1 var2=value2 \
                --authorize --description "Sample group description" --detect false --output json'
            var_group_output = self.cmd(create_variable_group_command).get_output_in_json()
            assert var_group_output["id"] > 0
            assert var_group_output["name"] == 'group1'
            assert var_group_output["description"] == "Sample group description"
            assert var_group_output["authorized"] == True
            assert len(var_group_output["variables"]) == 2
            assert var_group_output["variables"]["var1"]["value"] == "value1"
            assert var_group_output["variables"]["var2"]["value"] == "value2"
            group_id1 = var_group_output["id"]
            
            create_variable_group_command = 'az pipelines variable-group create --name group2 \
                --variables var1=value1 var2=value2 \
                --authorize --description "Sample group description" --detect false --output json'
            var_group_output = self.cmd(create_variable_group_command).get_output_in_json()
            assert var_group_output["id"] > 0
            assert var_group_output["name"] == 'group2'
            group_id2 = var_group_output["id"]

            # List variable groups
            list_variable_group_command = 'az pipelines variable-group list --detect false --output json'
            list_var_group_output = self.cmd(list_variable_group_command).get_output_in_json()
            assert len(list_var_group_output) == 2
            assert list_var_group_output[0]["name"] == 'group2' # since default order is desc
            assert list_var_group_output[1]["name"] == 'group1'

            # show variable group
            show_variable_group_command = 'az pipelines variable-group show --id {} --detect false --output json'.format(group_id1)
            show_var_group_output = self.cmd(show_variable_group_command).get_output_in_json()
            assert show_var_group_output["id"] == group_id1
            assert show_var_group_output["name"] == 'group1'
            assert show_var_group_output["description"] == "Sample group description"
            assert show_var_group_output["authorized"] == True
            assert len(show_var_group_output["variables"]) == 2
            assert show_var_group_output["variables"]["var1"]["value"] == "value1"
            assert show_var_group_output["variables"]["var2"]["value"] == "value2"

            # delete variable group
            delete_variable_group_command = 'az pipelines variable-group delete --id {} --detect false --output json -y'.format(group_id1)
            delete_var_group_output = self.cmd(delete_variable_group_command)
            # verify deletion
            # List variable groups
            list_variable_group_command = 'az pipelines variable-group list --detect false --output json'
            list_var_group_output = self.cmd(list_variable_group_command).get_output_in_json()
            assert len(list_var_group_output) == 1
            assert list_var_group_output[0]["name"] == 'group2'

            # update variable group
            update_variable_group_command = 'az pipelines variable-group update --name newGroup \
                --description NewDescription --authorize false --id {} --detect false --output json'.format(group_id2)
            update_var_group_output = self.cmd(update_variable_group_command).get_output_in_json()
            assert update_var_group_output["id"] == group_id2
            assert update_var_group_output["name"] == 'newGroup'
            assert update_var_group_output["description"] == "NewDescription"

            # List variables in the group 
            list_variable_group_vars_command = 'az pipelines variable-group variable list --id {} \
                 --detect false --output json'.format(group_id2)
            list_var_group_vars_output = self.cmd(list_variable_group_vars_command).get_output_in_json()
            assert len(list_var_group_vars_output) == 2
            assert list_var_group_vars_output['var1']['value'] == 'value1'
            assert list_var_group_vars_output['var2']['value'] == 'value2'

            # Add variable to the group
            add_variable_group_vars_command = 'az pipelines variable-group variable create --id {} --name NewVar1 --value NewVal1 \
                 --detect false --output json'.format(group_id2)
            add_var_group_vars_output = self.cmd(add_variable_group_vars_command).get_output_in_json()
            assert len(add_var_group_vars_output) == 1
            assert add_var_group_vars_output.get('var1') == None
            assert add_var_group_vars_output.get('var2') == None
            assert add_var_group_vars_output['NewVar1']['value'] == 'NewVal1'
            assert add_var_group_vars_output['NewVar1']['isSecret'] == None

            # Add a secret variable to the group
            add_variable_group_vars_command = 'az pipelines variable-group variable create --id {} --name NewVar2 --value NewVal2 \
                 --secret --detect false --output json'.format(group_id2)
            add_var_group_vars_output = self.cmd(add_variable_group_vars_command).get_output_in_json()
            assert len(add_var_group_vars_output) == 1
            assert add_var_group_vars_output.get('var1') == None
            assert add_var_group_vars_output.get('var2') == None
            assert add_var_group_vars_output.get('NewVar1') == None
            assert add_var_group_vars_output['NewVar2']['value'] == None
            assert add_var_group_vars_output['NewVar2']['isSecret'] == True

            # update variable in the group 
            update_variable_group_vars_command = 'az pipelines variable-group variable update --id {} --name NewVar1 \
                 --new-name NewVar1Updated --secret true --value 1234 --detect false --output json'.format(group_id2)
            update_var_group_vars_output = self.cmd(update_variable_group_vars_command).get_output_in_json()
            assert len(update_var_group_vars_output) == 1
            assert update_var_group_vars_output.get('var1') == None
            assert update_var_group_vars_output.get('var2') == None
            assert update_var_group_vars_output.get('NewVar2') == None
            assert update_var_group_vars_output['NewVar1Updated']['value'] == None
            assert update_var_group_vars_output['NewVar1Updated']['isSecret'] == True

            # delete the variable from the group
            delete_variable_group_vars_command = 'az pipelines variable-group variable delete --id {} \
                 --name var1 --detect false --output json -y'.format(group_id2)
            delete_var_group_vars_output = self.cmd(delete_variable_group_vars_command)

            # list to verify update and delete 
            list_variable_group_vars_command = 'az pipelines variable-group variable list --id {} \
                 --detect false --output json'.format(group_id2)
            list_var_group_vars_output = self.cmd(list_variable_group_vars_command).get_output_in_json()
            assert len(list_var_group_vars_output) == 3
            assert list_var_group_vars_output['var2']['value'] == 'value2'
            assert list_var_group_vars_output['NewVar1Updated']['value'] == None
            assert list_var_group_vars_output['NewVar1Updated']['isSecret'] == True
            assert list_var_group_vars_output['NewVar2']['value'] == None
            assert list_var_group_vars_output['NewVar2']['isSecret'] == True

        finally:
            if created_project_id is not None:
                delete_project_command = 'az devops project delete --id ' + created_project_id + ' --output json --detect false -y'
                self.cmd(delete_project_command)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class ContributionsClient(Client):
    """Contributions
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(ContributionsClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = '8477aec9-a4c7-4bd4-a456-ba4c53c989cb'

    def query_contribution_nodes(self, query):
        """QueryContributionNodes.
        [Preview API] Query for contribution nodes and provider details according the parameters in the passed in query object.
        :param :class:`<ContributionNodeQuery> <azure.devops.v5_0.contributions.models.ContributionNodeQuery>` query:
        :rtype: :class:`<ContributionNodeQueryResult> <azure.devops.v5_0.contributions.models.ContributionNodeQueryResult>`
        """
        content = self._serialize.body(query, 'ContributionNodeQuery')
        response = self._send(http_method='POST',
                              location_id='db7f2146-2309-4cee-b39c-c767777a1c55',
                              version='5.0-preview.1',
                              content=content)
        return self._deserialize('ContributionNodeQueryResult', response)

    def query_data_providers(self, query, scope_name=None, scope_value=None):
        """QueryDataProviders.
        [Preview API]
        :param :class:`<DataProviderQuery> <azure.devops.v5_0.contributions.models.DataProviderQuery>` query:
        :param str scope_name:
        :param str scope_value:
        :rtype: :class:`<DataProviderResult> <azure.devops.v5_0.contributions.models.DataProviderResult>`
        """
        route_values = {}
        if scope_name is not None:
            route_values['scopeName'] = self._serialize.url('scope_name', scope_name, 'str')
        if scope_value is not None:
            route_values['scopeValue'] = self._serialize.url('scope_value', scope_value, 'str')
        content = self._serialize.body(query, 'DataProviderQuery')
        response = self._send(http_method='POST',
                              location_id='738368db-35ee-4b85-9f94-77ed34af2b0d',
                              version='5.0-preview.1',
                              route_values=route_values,
                              content=content)
        return self._deserialize('DataProviderResult', response)

    def get_installed_extensions(self, contribution_ids=None, include_disabled_apps=None, asset_types=None):
        """GetInstalledExtensions.
        [Preview API]
        :param [str] contribution_ids:
        :param bool include_disabled_apps:
        :param [str] asset_types:
        :rtype: [InstalledExtension]
        """
        query_parameters = {}
        if contribution_ids is not None:
            contribution_ids = ";".join(contribution_ids)
            query_parameters['contributionIds'] = self._serialize.query('contribution_ids', contribution_ids, 'str')
        if include_disabled_apps is not None:
            query_parameters['includeDisabledApps'] = self._serialize.query('include_disabled_apps', include_disabled_apps, 'bool')
        if asset_types is not None:
            asset_types = ":".join(asset_types)
            query_parameters['assetTypes'] = self._serialize.query('asset_types', asset_types, 'str')
        response = self._send(http_method='GET',
                              location_id='2648442b-fd63-4b9a-902f-0c913510f139',
                              version='5.0-preview.1',
                              query_parameters=query_parameters)
        return self._deserialize('[InstalledExtension]', self._unwrap_collection(response))

    def get_installed_extension_by_name(self, publisher_name, extension_name, asset_types=None):
        """GetInstalledExtensionByName.
        [Preview API]
        :param str publisher_name:
        :param str extension_name:
        :param [str] asset_types:
        :rtype: :class:`<InstalledExtension> <azure.devops.v5_0.contributions.models.InstalledExtension>`
        """
        route_values = {}
        if publisher_name is not None:
            route_values['publisherName'] = self._serialize.url('publisher_name', publisher_name, 'str')
        if extension_name is not None:
            route_values['extensionName'] = self._serialize.url('extension_name', extension_name, 'str')
        query_parameters = {}
        if asset_types is not None:
            asset_types = ":".join(asset_types)
            query_parameters['assetTypes'] = self._serialize.query('asset_types', asset_types, 'str')
        response = self._send(http_method='GET',
                              location_id='3e2f6668-0798-4dcb-b592-bfe2fa57fde2',
                              version='5.0-preview.1',
                              route_values=route_values,
                              query_parameters=query_parameters)
        return self._deserialize('InstalledExtension', response)




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class BoardsClient(Client):
    """Boards
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(BoardsClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = '11635d5f-a4f9-43ea-a48b-d56be43fee0f'




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest import Serializer, Deserializer
from ...client import Client
from . import models


class ProfileRegionsClient(Client):
    """ProfileRegions
    :param str base_url: Service URL
    :param Authentication creds: Authenticated credentials.
    """

    def __init__(self, base_url=None, creds=None):
        super(ProfileRegionsClient, self).__init__(base_url, creds)
        client_models = {k: v for k, v in models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)

    resource_area_identifier = '8ccfef3d-2b87-4e99-8ccb-66e343d2daa8'

    def get_geo_region(self, ip):
        """GetGeoRegion.
        [Preview API] Lookup up country/region based on provided IPv4, null if using the remote IPv4 address.
        :param str ip:
        :rtype: :class:`<GeoRegion> <azure.devops.v6_0.profile_regions.models.GeoRegion>`
        """
        query_parameters = {}
        if ip is not None:
            query_parameters['ip'] = self._serialize.query('ip', ip, 'str')
        response = self._send(http_method='GET',
                              location_id='35b3ff1d-ab4c-4d1c-98bb-f6ea21d86bd9',
                              version='6.0-preview.1',
                              query_parameters=query_parameters)
        return self._deserialize('GeoRegion', response)

    def get_regions(self):
        """GetRegions.
        [Preview API]
        :rtype: :class:`<ProfileRegions> <azure.devops.v6_0.profile_regions.models.ProfileRegions>`
        """
        response = self._send(http_method='GET',
                              location_id='b129ca90-999d-47bb-ab37-0dcf784ee633',
                              version='6.0-preview.1')
        return self._deserialize('ProfileRegions', response)




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from collections import OrderedDict


def transform_banner_table_output(result):
    table_output = []
    for k, v in result.items():
        table_output.append(_transform_banner_row(k, v))
    return table_output


def _transform_banner_row(key, value):
    table_row = OrderedDict()
    table_row['ID'] = key
    if 'message' in value:
        table_row['Message'] = value['message']
    else:
        table_row['Message'] = ' '
    if 'level' in value:
        level = value['level']
        if level is not None and level != '':
            level = level[0:1].upper() + level[1:]
        table_row['Type'] = level
    else:
        table_row['Type'] = 'Info'
    if 'expirationDate' in value:
        table_row['Expiration Date'] = value['expirationDate']
    else:
        table_row['Expiration Date'] = ' '
    return table_row



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# Generated file, DO NOT EDIT
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------------------------

from msrest.serialization import Model


class SessionRequest(Model):
    """
    :param data: Generic property bag to store data about the session
    :type data: dict
    :param feed: The feed name or id for the session
    :type feed: str
    :param source: The type of session If a known value is provided, the Data dictionary will be validated for the presence of properties required by that type
    :type source: str
    """

    _attribute_map = {
        'data': {'key': 'data', 'type': '{str}'},
        'feed': {'key': 'feed', 'type': 'str'},
        'source': {'key': 'source', 'type': 'str'}
    }

    def __init__(self, data=None, feed=None, source=None):
        super(SessionRequest, self).__init__()
        self.data = data
        self.feed = feed
        self.source = source


class SessionResponse(Model):
    """
    :param session_id: The unique identifier for the session
    :type session_id: str
    :param session_name: The name for the session
    :type session_name: str
    """

    _attribute_map = {
        'session_id': {'key': 'sessionId', 'type': 'str'},
        'session_name': {'key': 'sessionName', 'type': 'str'}
    }

    def __init__(self, session_id=None, session_name=None):
        super(SessionResponse, self).__init__()
        self.session_id = session_id
        self.session_name = session_name


__all__ = [
    'SessionRequest',
    'SessionResponse',
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import subprocess
import sys

from knack.log import get_logger
from knack.util import CLIError
from .uri import uri_parse

logger = get_logger(__name__)

_GIT_EXE = 'git'


def set_config(key, value, local=True):
    scope = _get_git_config_scope_arg(local)
    subprocess.check_output([_GIT_EXE, 'config', scope, key, value])


def unset_config(key, local=True):
    scope = _get_git_config_scope_arg(local)
    subprocess.check_output([_GIT_EXE, 'config', scope, '--unset', key])


def get_config(key, local=True):
    scope = _get_git_config_scope_arg(local)
    return subprocess.check_output([_GIT_EXE, 'config', scope, key])


def _get_git_config_scope_arg(local):
    if local:
        return '--local'
    return '--global'


def fetch_remote_and_checkout(refName, remote_name):
    subprocess.run([_GIT_EXE, 'fetch', remote_name, refName], check=False)
    subprocess.run([_GIT_EXE, 'checkout', get_branch_name_from_ref(refName)], check=False)
    subprocess.run([_GIT_EXE, 'pull', remote_name, get_branch_name_from_ref(refName)], check=False)


def get_current_branch_name():
    try:
        output = subprocess.check_output([_GIT_EXE, 'symbolic-ref', '--short', '-q', 'HEAD'])
    except BaseException as ex:  # pylint: disable=broad-except
        logger.info('GitDetect: Could not detect current branch based on current working directory.')
        logger.debug(ex, exc_info=True)
        return None
    if sys.stdout.encoding is not None:
        result = output.decode(sys.stdout.encoding)
    else:
        result = output.decode()
    return result.strip()


def get_remote_url(validation_function=None):
    remotes = get_git_remotes()
    if remotes is not None:
        if _ORIGIN_PUSH_KEY in remotes and (validation_function is None or
                                            validation_function(remotes[_ORIGIN_PUSH_KEY])):
            return remotes[_ORIGIN_PUSH_KEY]
        for k, value in remotes.items():
            if k != _ORIGIN_PUSH_KEY and k.endswith('(push)') and (validation_function is None or
                                                                   validation_function(value)):
                return value
    return None


def get_git_credentials(organization):
    parse_result = uri_parse(organization)
    protocol = parse_result.scheme
    host = parse_result.netloc
    standard_in = bytes('protocol={protocol}\nhost={host}'.format(protocol=protocol, host=host), 'utf-8')
    try:
        # pylint: disable=unexpected-keyword-arg
        output = subprocess.check_output([_GIT_EXE, 'credential-manager', 'get'], input=standard_in)
    except BaseException as ex:  # pylint: disable=broad-except
        logger.info('GitDetect: Could not detect git credentials for current working directory.')
        logger.debug(ex, exc_info=True)
        return None
    if sys.stdout.encoding is not None:
        lines = output.decode(sys.stdout.encoding).split('\n')
    else:
        lines = output.decode().split('\n')
    properties = {}
    for line in lines:
        equal_position = line.find('=')
        if equal_position >= 0:
            properties[line[0:equal_position]] = line[equal_position + 1:]
    return properties


def get_git_remotes():
    if _git_remotes:
        return _git_remotes
    try:
        # Example output:
        # git remote - v
        # full  https://mseng.visualstudio.com/DefaultCollection/VSOnline/_git/_full/VSO (fetch)
        # full  https://mseng.visualstudio.com/DefaultCollection/VSOnline/_git/_full/VSO (push)
        # origin  https://mseng.visualstudio.com/defaultcollection/VSOnline/_git/VSO (fetch)
        # origin  https://mseng.visualstudio.com/defaultcollection/VSOnline/_git/VSO (push)
        output = subprocess.check_output([_GIT_EXE, 'remote', '-v'], stderr=subprocess.STDOUT)
    except BaseException as ex:  # pylint: disable=broad-except
        logger.info('GitDetect: Could not detect current remotes based on current working directory.')
        logger.debug(ex, exc_info=True)
        return None
    if sys.stdout.encoding is not None:
        lines = output.decode(sys.stdout.encoding).split('\n')
    else:
        lines = output.decode().split('\n')
    for line in lines:
        components = line.strip().split()
        if len(components) == 3:
            _git_remotes[components[0] + components[2]] = components[1]
    return _git_remotes


def resolve_git_refs(ref):
    """Prepends 'refs/' prefix to ref str if not already there.
    :param str ref: The text to validate.
    :rtype: str
    """
    if ref is not None and not ref.startswith(REFS_PREFIX):
        ref = REFS_PREFIX + ref
    return ref


def get_ref_name_from_ref(ref):
    """Removes 'refs/' prefix from ref str if there.
    :param ref: The text to validate.
    :type ref: str
    :rtype: str
    """
    if ref is not None and ref.startswith(REFS_PREFIX):
        ref = ref[len(REFS_PREFIX):]
    return ref


def resolve_git_ref_heads(ref):
    """Prepends 'refs/heads/' prefix to ref str if not already there.
    :param ref: The text to validate.
    :type ref: str
    :rtype: str
    """
    if (ref is not None and not ref.startswith(REF_HEADS_PREFIX) and
            not ref.startswith(REF_PULL_PREFIX) and not ref.startswith(REF_TAGS_PREFIX)):
        ref = REF_HEADS_PREFIX + ref
    return ref


def get_branch_name_from_ref(ref):
    """Removes 'refs/heads/' prefix from ref str if there.
    :param ref: The text to validate.
    :type ref: str
    :rtype: str
    """
    if ref is not None and ref.startswith(REF_HEADS_PREFIX):
        ref = ref[len(REF_HEADS_PREFIX):]
    return ref


def setup_git_alias(alias, command, local=False):
    try:
        set_config(key=_get_alias_key(alias),
                   value=_get_alias_value(command),
                   local=local)
    except OSError:
        raise CLIError('Setting the git alias failed. Ensure git is installed and in your path.')


def clear_git_alias(alias, local=False):
    unset_config(key=_get_alias_key(alias), local=local)


def is_git_alias_setup(alias, command, local=False):
    try:
        try:
            value = get_config(key=_get_alias_key(alias), local=local)
        except subprocess.CalledProcessError:
            return False
        return _get_alias_value(command) == value.decode(sys.stdout.encoding).strip()
    except OSError:
        raise CLIError('Checking the git config values failed. Ensure git is installed and in your path.')


def _get_alias_key(alias):
    return 'alias.' + alias


def _get_alias_value(command):
    mime = '.cmd' if sys.platform.lower().startswith('win') else ''
    return '!f() { exec az' + mime + ' ' + command + ' \"$@\"; }; f'


_git_remotes = {}
_ORIGIN_PUSH_KEY = 'origin(push)'
REFS_PREFIX = 'refs/'
REF_HEADS_PREFIX = 'refs/heads/'
REF_PULL_PREFIX = 'refs/pull/'
REF_TAGS_PREFIX = 'refs/tags/'
GIT_CREDENTIALS_USERNAME_KEY = 'username'
GIT_CREDENTIALS_PASSWORD_KEY = 'password'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import json
import unittest

try:
    # Attempt to load mock (works on Python 3.3 and above)
    from unittest.mock import patch
except ImportError:
    # Attempt to load mock (works on Python version below 3.3)
    from mock import patch

from azext_devops.devops_sdk.v5_0.security.security_client import (SecurityClient)
from azext_devops.devops_sdk.v5_0.security.models import (AccessControlList,
                                                          AccessControlEntry,
                                                          AceExtendedInformation,
                                                          ActionDefinition,
                                                          SecurityNamespaceDescription)
from azext_devops.dev.team.security_permission import (list_namespaces,
                                                       show_namespace,
                                                       list_tokens,
                                                       show_permissions,
                                                       reset_all_permissions,
                                                       reset_permissions,
                                                       update_permissions,
                                                       _resolve_bits)

from azext_devops.dev.common.services import clear_connection_cache
from azext_devops.test.utils.authentication import AuthenticatedTests
from azext_devops.test.utils.helper import get_client_mock_helper

class TestSecurityPermissionsMethods(AuthenticatedTests):
    _TEST_DEVOPS_ORGANIZATION = 'https://someorganization.visualstudio.com'
    _SECURITY_CLIENT_LOCATION = 'azext_devops.devops_sdk.v5_0.security.security_client.SecurityClient.'
    _TEST_GROUP_DESCRIPTOR = 'vssgp.someRandomDescriptorForGroup'
    _TEST_GROUP_NAME = 'New test security group'
    _TEST_SECURITY_TOKEN = 'some_security_token'
    _TEST_IDENTITY_DESCRIPTOR = 'Microsoft.Identity.aaaaaaaa'
    _TEST_SECURITY_NAMESPACE_ID = 'bf7bfa03-b2b7-47db-xxxx-fa2e002xxxx'

    def setUp(self):
        self.authentication_setup()
        self.authenticate()
        
        self.list_namespaces_patcher = patch(self._SECURITY_CLIENT_LOCATION + 'query_security_namespaces')
        self.list_acl_patcher = patch(self._SECURITY_CLIENT_LOCATION + 'query_access_control_lists')
        self.remove_ace_patcher = patch(self._SECURITY_CLIENT_LOCATION + 'remove_access_control_entries')
        self.set_ace_patcher = patch(self._SECURITY_CLIENT_LOCATION + 'set_access_control_entries')
        self.remove_permission_patcher = patch(self._SECURITY_CLIENT_LOCATION + 'remove_permission')
        self.get_identity_descriptor_patcher = patch('azext_devops.dev.team.security_permission._resolve_subject_as_identity_descriptor')
        self.resolve_bits_patcher = patch('azext_devops.dev.team.security_permission._resolve_bits')
        self.get_client = patch('azext_devops.devops_sdk.connection.Connection.get_client')

        self.mock_get_client = self.get_client.start()
        self.mock_get_identity_descriptor = self.get_identity_descriptor_patcher.start()
        self.mock_list_namespaces = self.list_namespaces_patcher.start()
        self.mock_list_acl = self.list_acl_patcher.start()
        self.mock_resolve_bits = self.resolve_bits_patcher.start()
        self.mock_set_ace = self.set_ace_patcher.start()
        self.mock_remove_permission = self.remove_permission_patcher.start()
        self.mock_remove_ace = self.remove_ace_patcher.start()
        #set return values
        self.mock_get_client.return_value = SecurityClient(base_url=self._TEST_DEVOPS_ORGANIZATION)
        self.mock_get_identity_descriptor.return_value = self._TEST_IDENTITY_DESCRIPTOR
        #clear connection cache before running each test
        clear_connection_cache()

    def tearDown(self):
        patch.stopall()

    def test_list_namespaces(self):
        response = list_namespaces(False,organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_list_namespaces.assert_called_once()
        list_namespaces_param = self.mock_list_namespaces.call_args_list[0][1]
        self.assertEqual(False, list_namespaces_param['local_only'])
    
    def test_show_namespace(self):
        response = show_namespace(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_list_namespaces.assert_called_once()
        show_namespace_param = self.mock_list_namespaces.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, show_namespace_param['security_namespace_id'])
    
    def test_list_tokens(self):
        response = list_tokens(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,
                                subject = self._TEST_GROUP_DESCRIPTOR, 
                                recurse=False,organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_list_acl.assert_called_once()
        list_tokens_param = self.mock_list_acl.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, list_tokens_param['security_namespace_id'])
        self.assertEqual(False, list_tokens_param['recurse'])
        self.assertEqual(True, list_tokens_param['include_extended_info'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, list_tokens_param['descriptors'])
        
    def test_show_permissions(self):
        self.mock_resolve_bits.return_value = self._get_resolve_bits_response()
        response = show_permissions(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,
                                    subject = self._TEST_GROUP_DESCRIPTOR, 
                                    token= self._TEST_SECURITY_TOKEN,
                                    organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_list_acl.assert_called_once()
        show_permissions_param = self.mock_list_acl.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, show_permissions_param['security_namespace_id'])
        self.assertEqual(False, show_permissions_param['recurse'])
        self.assertEqual(self._TEST_SECURITY_TOKEN, show_permissions_param['token'])
        self.assertEqual(True, show_permissions_param['include_extended_info'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, show_permissions_param['descriptors'])

    def test_update_permissions(self):
        self.mock_resolve_bits.return_value = self._get_resolve_bits_response()       
        response = update_permissions(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,
                                    allow_bit = 3,
                                    subject = self._TEST_GROUP_DESCRIPTOR, 
                                    token= self._TEST_SECURITY_TOKEN,
                                    organization=self._TEST_DEVOPS_ORGANIZATION,
                                    merge=True)
        #assert
        self.mock_list_acl.assert_called_once()
        self.mock_set_ace.assert_called_once()
        show_permissions_param = self.mock_list_acl.call_args_list[0][1]
        update_permissions_param = self.mock_set_ace.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_TOKEN, show_permissions_param['token'])
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, show_permissions_param['security_namespace_id'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, show_permissions_param['descriptors'])
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, update_permissions_param['security_namespace_id'])

    def test_reset_permissions(self):
        self.mock_resolve_bits.return_value = self._get_resolve_bits_response()
        response = reset_permissions(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,
                                    permission_bit = 3,
                                    subject = self._TEST_GROUP_DESCRIPTOR, 
                                    token= self._TEST_SECURITY_TOKEN,
                                    organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_list_acl.assert_called_once()
        self.mock_remove_permission.assert_called_once()
        show_permissions_param = self.mock_list_acl.call_args_list[0][1]
        remove_permissions_param = self.mock_remove_permission.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_TOKEN, show_permissions_param['token'])
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, show_permissions_param['security_namespace_id'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, show_permissions_param['descriptors'])
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, remove_permissions_param['security_namespace_id'])
        self.assertEqual(3, remove_permissions_param['permissions'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, remove_permissions_param['descriptor'])
        self.assertEqual(self._TEST_SECURITY_TOKEN, remove_permissions_param['token'])

    def test_reset_all_permissions(self):
        response = reset_all_permissions(namespace_id=self._TEST_SECURITY_NAMESPACE_ID,
                                         subject = self._TEST_GROUP_DESCRIPTOR,
                                         token = self._TEST_SECURITY_TOKEN,
                                         organization=self._TEST_DEVOPS_ORGANIZATION)
        #assert
        self.mock_remove_ace.assert_called_once()
        remove_ace_param = self.mock_remove_ace.call_args_list[0][1]
        self.assertEqual(self._TEST_SECURITY_NAMESPACE_ID, remove_ace_param['security_namespace_id'])
        self.assertEqual(self._TEST_SECURITY_TOKEN, remove_ace_param['token'])
        self.assertEqual(self._TEST_IDENTITY_DESCRIPTOR, remove_ace_param['descriptors'])

    def _get_resolve_bits_response(self,changed_bits=0):
        # api response
        namespace_details = self._create_namespace_details_response()
        acl_response = self._create_permission_api_response()
        resolved_bits_response = _resolve_bits(acl_response, namespace_details,changed_bits=changed_bits)
        return resolved_bits_response
    
    def _create_namespace_details_response(self):
        #namespace list reponse
        namespace_details = []
        namespace_details_entry = SecurityNamespaceDescription()
        dummy_actions = []
        # GENERIC_READ
        readAction = ActionDefinition()
        readAction.bit = 1
        readAction.displayName = 'View Resource'
        readAction.name = 'GENERIC_READ'
        readAction.namespace_id = self._TEST_SECURITY_NAMESPACE_ID
        dummy_actions.append(readAction)
        # GENERIC_WRITE
        writeAction = ActionDefinition()
        writeAction.bit = 2
        writeAction.displayName = 'Create Resource'
        writeAction.name = 'GENERIC_WRITE'
        writeAction.namespace_id = self._TEST_SECURITY_NAMESPACE_ID
        dummy_actions.append(writeAction)
        # GENERIC_UPDATE
        updateAction = ActionDefinition()
        updateAction.bit = 4
        updateAction.displayName = 'Update Resource'
        updateAction.name = 'GENERIC_UPDATE'
        updateAction.namespace_id = self._TEST_SECURITY_NAMESPACE_ID
        dummy_actions.append(updateAction)
        namespace_details_entry.actions = dummy_actions
        namespace_details.append(namespace_details_entry)
        return namespace_details

    def _create_permission_api_response(self, allow_bit=0, deny_bit=0, effective_allow=0, effective_deny=0):
        acl_response = []
        acl_entry = AccessControlList()
        ace_dictionary = {}
        ace_entry = AccessControlEntry()
        ace_entry.allow = allow_bit
        ace_entry.deny = deny_bit
        ace_entry.descriptor = self._TEST_IDENTITY_DESCRIPTOR
        extended_info = AceExtendedInformation()
        extended_info.effective_allow = effective_allow
        extended_info.effective_deny = effective_deny
        ace_entry.extended_info = extended_info
        ace_dictionary[self._TEST_IDENTITY_DESCRIPTOR] = ace_entry
        acl_entry.aces_dictionary = ace_dictionary
        acl_entry.include_extended_info = True
        acl_entry.inherit_permissions = False
        acl_entry.token = self._TEST_SECURITY_TOKEN
        acl_response.append(acl_entry)
        return acl_response

